package de.dfki.sds.hephaistos.server;

import de.dfki.sds.hephaistos.storage.StorageManager;
import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.output.WriterOutputStream;
import org.apache.http.entity.ContentType;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.Query;
import org.apache.jena.query.QueryExecution;
import org.apache.jena.query.QueryExecutionFactory;
import org.apache.jena.query.ResultSet;
import org.apache.jena.query.ResultSetFormatter;
import org.apache.jena.system.Txn;
import org.eclipse.jetty.http.MimeTypes;
import org.json.JSONObject;
import spark.Request;
import spark.Response;
import spark.Spark;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class Server {
    
    //HePhaistos
    public static final int PORT = (int) 'H' * 100 + (int) 'P';
    public static final String HOST = "http://localhost:" + PORT;
    
    private static final String ROOT_PATH = "/de/dfki/sds/hephaistos";
    
    private StorageManager storageManager;
    
    public Server() {
        initSpark();
        initRoutes();
    }
    
    public void close() {
        Spark.stop();
        Spark.awaitStop();
    }
    
    private void initSpark() {
        System.out.println("port: " + PORT);
        Spark.port(PORT);
        
        Spark.exception(Exception.class, (exception, request, response) -> {
            exception.printStackTrace();
            response.body(exception.getMessage());
        });
        
        Spark.staticFiles.location(ROOT_PATH + "/web");
        
        //if ends with '/' redirect to path without '/'
        Spark.before((req, res) -> {
            String path = req.pathInfo();
            if (!path.equals("/") && path.endsWith("/")) {
                res.redirect(path.substring(0, path.length() - 1));
            }
        });
        
        Spark.awaitInitialization();
    }
    
    private void initRoutes() {
        Spark.get("/", this::getRoot);
        Spark.get("/favicon.ico", this::getFavicon);
        //TODO getOntology
    
        //in setProject are the spark routes for /project
        //in setProject are the sparql routes
    }
    
    private Object getRoot(Request req, Response resp) {
        
        
        return null;
    }

    private Object getFavicon(Request req, Response resp) throws IOException {
        resp.type(ContentType.IMAGE_PNG.toString());
        return IOUtils.toByteArray(Server.class.getResourceAsStream("/de/dfki/sds/hephaistos/gui/ontology.png"));
    }

    /*
    public void setProject(Project project) {
        this.project = project;
        
        //for deep links
        Spark.get("/" + Project.PROJECT_SEG, this::getProject);
        Spark.get("/" + Project.PROJECT_SEG + "/*", this::getProject);
        Spark.get("/" + Project.PROJECT_SEG + "/**", this::getProject);
        
        setStorageManager(project.getStorageManager());
    }
    */
    
    /*
    private Object getProject(Request req, Response resp) throws Exception {
        //uri = the path (without parameters)
        DeepLink dl = project.getDeepLinker().processPath(req.uri());
        
        resp.type("text/turtle");
        
        Model model = ModelFactory.createDefaultModel();
        model.setNsPrefixes(PrefixMapping.Standard);
        model.setNsPrefix("", Server.HOST);
        model.setNsPrefix("onto", Server.HOST + "/ontology/");
        
        if(dl.last() instanceof FIToRDF) {
            FIToRDF fragmentIdent = (FIToRDF) dl.last();
            fragmentIdent.toRDF(dl.asResource(), model, dl);
        }
        
        return DeepLinker.toTurtle(model);
    }
    */

    public StorageManager getStorageManager() {
        return storageManager;
    }

    public void setStorageManager(StorageManager storageManager) {
        this.storageManager = storageManager;
        
        //sparql
        Spark.get("/sparql", this::getSparql);
        Spark.post("/sparql", this::postSparql);
    }
    
    private Object getSparql(Request req, Response resp) throws IOException {
        return IOUtils.toString(Server.class.getResourceAsStream(ROOT_PATH + "/web/sparql.html"), StandardCharsets.UTF_8);
    }
    
    private Object postSparql(Request req, Response resp) {
        String query = req.queryParamOrDefault("query", "");
        
        Dataset dataset = storageManager.getRDFDatasetStorage().getDataset();
        
        QueryExecution qe = QueryExecutionFactory.create(query,dataset);
        Query q = qe.getQuery();
        
        if(q.isSelectType()) {
        
            ResultSet rs = qe.execSelect();

            //run and convert to json
            JSONObject result = Txn.calculateRead(dataset, () -> {
                StringWriter sw = new StringWriter();
                OutputStream out = new WriterOutputStream(sw, StandardCharsets.UTF_8);
                ResultSetFormatter.outputAsJSON(out, rs);
                return new JSONObject(sw.toString());
            });

            resp.type(MimeTypes.Type.APPLICATION_JSON.asString());
            return result.toString(2);
        
        } else {
            return "query type not supported: " + q.getQueryType();
        }
    }
    
}
