package de.dfki.sds.hephaistos.storage;

import de.dfki.sds.hephaistos.Counter;
import de.dfki.sds.hephaistos.storage.rdf.RDFStorageItem;
import java.util.List;
import org.apache.jena.query.Dataset;
import org.apache.jena.query.ResultSet;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;

/**
 * Manages named graphs.
 * @author Markus Schr&ouml;der
 */
public abstract class RDFDatasetStorage extends InternalStorage<RDFStorageItem, StorageSummary, ResultSet> {
    
    public static final String DEEPLINK = StorageManager.DEEPLINK + "/" + StorageManager.KG_RES_SEG;
    
    public RDFDatasetStorage(InternalStorageMetaData metaData) {
        super(metaData);
    }
    
    public abstract void addGraph(String uri);
    
    public abstract void removeGraph(String uri);
    
    public abstract boolean containsGraph(String uri);
    
    public abstract List<String> getGraphNames();
    
    public abstract Dataset getDataset();

    /**
     * Creates a knowledge resource with an counter.
     * Uses {@link StorageManager#KG_RES_SEG} as counter name.
     * Uses the {@link #DEEPLINK}.
     * @param c the counter instance
     * @return a fresh KG resource
     */
    public static Resource create(Counter c) {
        return ResourceFactory.createResource(DEEPLINK + "/" + c.getIncreased(StorageManager.KG_RES_SEG));
    }
}
