package de.dfki.sds.hephaistos.storage;

import de.dfki.sds.hephaistos.storage.rdf.RDFGraphStorageSummary;
import de.dfki.sds.hephaistos.storage.rdf.RDFStorageItem;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.function.Supplier;
import org.apache.commons.io.FileUtils;
import org.apache.jena.query.ResultSet;
import org.apache.jena.rdf.model.Model;

/**
 * An RDF graph storage for knowledge graph and other graph-like RDF data stores.
 * @author Markus Schr&ouml;der
 */
public abstract class RDFGraphStorage extends InternalStorage<RDFStorageItem, RDFGraphStorageSummary, ResultSet> {
    
    public RDFGraphStorage(InternalStorageMetaData metaData) {
        super(metaData);
    }
    
    public abstract Model getModel();
    
    
    
    public abstract void executeRead(Runnable r);
    
    public abstract void executeWrite(Runnable r);
    
    public abstract <T> T calculateRead(Supplier<T> s);
    
    public abstract <T> T calculateWrite(Supplier<T> s);
    
    
    //every graph can have a name and a prefix
    //the id is the graph's URI
    
    public abstract void setLabel(String label);
    
    public abstract String getLabel();
    
    public abstract void setPrefix(String prefix);
    
    public abstract String getPrefix();
    
    
    
    public String toTTL() {
        return calculateRead(() -> {
            StringWriter sw = new StringWriter();
            getModel().write(sw, "TTL");
            return sw.toString();
        });
    }
    
    public void printTTL() {
        System.out.println(toTTL());
    }
    
    public void saveTTL(File f) throws IOException {
        FileUtils.writeStringToFile(f, toTTL(), StandardCharsets.UTF_8);
    }
}
