package de.dfki.sds.hephaistos.storage.dsv;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import org.apache.commons.csv.CSVRecord;

/**
 * Maps header names to column index.
 * @author Markus Schr&ouml;der
 */
public class DSVHeader extends HashMap<String, Integer> {

    public DSVHeader() {
    }

    public DSVHeader(Map<? extends String, ? extends Integer> m) {
        super(m);
    }
    
    public DSVHeader(CSVRecord csvRecord) {
        this(csvRecord.getParser().getHeaderMap());
    }
    
    public static boolean hasHeader(CSVRecord csvRecord) {
        return csvRecord.getParser().getHeaderMap() != null;
    }
    
    public DSVHeader(String... columnNames) {
        for(int i = 0; i < columnNames.length; i++) {
            put(columnNames[i], i);
        }
    }
    
    public DSVHeader(List<String> columnNames) {
        for(int i = 0; i < columnNames.size(); i++) {
            put(columnNames.get(i), i);
        }
    }
    
    public List<String> toList() {
        return this.entrySet()
                .stream()
                .sorted(Map.Entry.comparingByValue())
                .map(e -> e.getKey())
                .collect(Collectors.toList());
    }
    
}
