package de.dfki.sds.hephaistos.storage.dsv;

import de.dfki.sds.hephaistos.storage.StorageItem;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.apache.commons.csv.CSVRecord;

/**
 * A delimiter-separated value record (inspired by CSVRecord).
 * @author Markus Schr&ouml;der
 */
public class DSVRecord implements StorageItem {
    
    private String deepLink;
    
    private DSVHeader header;
    
    private long characterPosition;
    
    private long recordNumber = -1;
    
    private String comment;
    
    private String[] values;
    
    //you can select in the record a column to point to it (e.g. for search results)
    private List<Integer> selectedColumns;
    
    public DSVRecord() {
    }

    /**
     * Use this constructor when you just need to delete the record.
     * @param recordNumber 
     */
    public DSVRecord(long recordNumber) {
        this.recordNumber = recordNumber;
    }
    
    public DSVRecord(String[] values) {
        this.values = values;
    }
    
    public DSVRecord(long recordNumber, String[] values) {
        this.recordNumber = recordNumber;
        this.values = values;
    }
    
    public DSVRecord(DSVHeader header, String[] values) {
        this.header = header;
        this.values = values;
    }
    
    public DSVRecord(DSVHeader header, long recordNumber, String[] values) {
        this.header = header;
        this.recordNumber = recordNumber;
        this.values = values;
    }
    
    
    /**
     * Use this record to create a DSVRecord from CSVRecord.
     * @param header
     * @param csvRecord 
     */
    public DSVRecord(DSVHeader header, CSVRecord csvRecord) {
        this.characterPosition = csvRecord.getCharacterPosition();
        this.recordNumber = csvRecord.getRecordNumber();
        this.comment = csvRecord.getComment();
        
        this.values = new String[csvRecord.size()];
        for(int i = 0; i < values.length; i++) {
            this.values[i] = csvRecord.get(i);
        }
        
        this.header = header;
    }

    
    public String get(int i) {
        return values[i];
    }
    
    public String get(String name) {
        if(!hasHeader()) {
            throw new RuntimeException("no header information");
        }
        
        return values[getHeader().get(name)];
    }
    
    
    public DSVHeader getHeader() {
        return header;
    }

    public void setHeader(DSVHeader header) {
        this.header = header;
    }

    public long getCharacterPosition() {
        return characterPosition;
    }

    public void setCharacterPosition(long characterPosition) {
        this.characterPosition = characterPosition;
    }

    public long getRecordNumber() {
        return recordNumber;
    }

    public void setRecordNumber(long recordNumber) {
        this.recordNumber = recordNumber;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public String[] getValues() {
        return values;
    }

    public void setValues(String[] values) {
        this.values = values;
    }

    public List<Integer> getSelectedColumns() {
        if(selectedColumns == null) {
            this.selectedColumns = new ArrayList<>();
        }
        return selectedColumns;
    }

    public void setSelectedColumns(List<Integer> selectedColumns) {
        this.selectedColumns = selectedColumns;
    }

    public boolean hasHeader() {
        return header != null;
    }
    
    @Override
    public String toString() {
        return "DSVRecord{" + "header=" + header + ", recordNumber=" + recordNumber + ", values=" + Arrays.toString(values) + '}';
    }

    public int size() {
        return values.length;
    }

    /*package*/ void setDeepLink(String deepLink) {
        this.deepLink = deepLink;
    }
    
    @Override
    public String getDeepLink() {
        return deepLink;
    }

}
