package de.dfki.sds.hephaistos.storage.rdf;

import de.dfki.sds.hephaistos.storage.RDFGraphStorage;
import de.dfki.sds.hephaistos.storage.StorageSummary;
import de.dfki.sds.hephaistos.storage.StorageSummaryCache;
import java.util.Arrays;
import java.util.List;
import org.apache.jena.rdf.model.RDFNode;
import org.apache.jena.vocabulary.RDF;
import org.apache.jena.vocabulary.RDFS;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class RDFGraphStorageSummary extends StorageSummary {
    
    private RDFGraphStorage storage;

    public RDFGraphStorageSummary(RDFGraphStorage storage, StorageSummaryCache cache) {
        super(cache);
        this.storage = storage;
    }

    @Override
    public List<String> prepare() {
        getNumberOfTriples();
        getNumberOfClasses();
        getNumberOfProperties();
        getNumberOfSubclasses();
        getNumberOfSubproperties();
        getNumberOfLabels();
        getNumberOfComments();
        return Arrays.asList();
    }
    
    public long getNumberOfTriples() {
        return cache.computeLongIfAbsent("number of triples", c -> storage.size());
    }
    
    public long getNumberOfClasses() {
        return cache.computeLongIfAbsent("number of classes", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDF.type, RDFS.Class).toSet().size();
            })
        );
    }
    
    public long getNumberOfProperties() {
        return cache.computeLongIfAbsent("number of properties", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDF.type, RDF.Property).toSet().size();
            })
        );
    }
    
    public long getNumberOfLabels() {
        return cache.computeLongIfAbsent("number of labels", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDFS.label, (RDFNode) null).toSet().size();
            })
        );
    }
    
    public long getNumberOfComments() {
        return cache.computeLongIfAbsent("number of comments", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDFS.comment, (RDFNode) null).toSet().size();
            })
        );
    }
    
    public long getNumberOfSubclasses() {
        return cache.computeLongIfAbsent("number of subclasses", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDFS.subClassOf, (RDFNode) null).toSet().size();
            })
        );
    }
    
    public long getNumberOfSubproperties() {
        return cache.computeLongIfAbsent("number of subproperties", c -> storage.calculateRead(() -> {
                return (long) storage.getModel().listStatements(null, RDFS.subPropertyOf, (RDFNode) null).toSet().size();
            })
        );
    }
    
}
