package de.dfki.sds.hephaistos.storage.term;

import de.dfki.sds.hephaistos.storage.StorageItem;

/**
 * A class holding the term (a string) and various term metrics (e.g. TF-IDF).
 * @author Markus Schr&ouml;der
 */
public class TermMetrics implements StorageItem {
    
    private String deepLink;
    
    private int id;
    
    private String term;
    
    
    /**
     * Total Term Fequency (TTF).
     */
    private double ttf;
    
    /**
     * Average Term Fequency (ATF).
     */
    private double atf;
    
    /**
     * Inverse Document Frequency (IDF).
     */
    private double idf;
    
    /**
     * Average Term Fequency (ATF) * Inverse Document Frequency (IDF).
     */
    private double atf_idf;
    
    /**
     * Total Term Fequency (TTF) * Inverse Document Frequency (IDF).
     */
    private double ttf_idf;
    
    /**
     * Residual Inverse Document Frequency (RIDF).
     * Residual = übrigbleibend.
     */
    private double ridf;
    
    /**
     * Multi-word: C-Value.
     */
    private double cvalue;
    
    /**
     * Multi-word: RAKE score (only frequency).
     */
    private double rake_frequency;
    
    /**
     * Multi-word: RAKE score (only degree).
     */
    private double rake_degree;
    
    /**
     * Multi-word: RAKE score (degree & frequency).
     */
    private double rake_degree_frequency;

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    public String getTerm() {
        return term;
    }

    public void setTerm(String term) {
        this.term = term;
    }

    public double getTTF() {
        return ttf;
    }

    public void setTTF(double ttf) {
        this.ttf = ttf;
    }

    public double getATF() {
        return atf;
    }

    public void setATF(double atf) {
        this.atf = atf;
    }

    public double getIDF() {
        return idf;
    }

    public void setIDF(double idf) {
        this.idf = idf;
    }

    public double getATF_IDF() {
        return atf_idf;
    }

    public void setATF_IDF(double atf_idf) {
        this.atf_idf = atf_idf;
    }

    public double getTTF_IDF() {
        return ttf_idf;
    }

    public void setTTF_IDF(double ttf_idf) {
        this.ttf_idf = ttf_idf;
    }

    public double getRIDF() {
        return ridf;
    }

    public void setRIDF(double ridf) {
        this.ridf = ridf;
    }

    public double getCValue() {
        return cvalue;
    }

    public void setCValue(double cvalue) {
        this.cvalue = cvalue;
    }

    public double getRAKE_Frequency() {
        return rake_frequency;
    }

    public void setRAKE_Frequency(double rake_frequency) {
        this.rake_frequency = rake_frequency;
    }

    public double getRAKE_Degree() {
        return rake_degree;
    }

    public void setRAKE_Degree(double rake_degree) {
        this.rake_degree = rake_degree;
    }

    public double getRAKE_Degree_Frequency() {
        return rake_degree_frequency;
    }

    public void setRAKE_Degree_Frequency(double rake_degree_frequency) {
        this.rake_degree_frequency = rake_degree_frequency;
    }

    @Override
    public String getDeepLink() {
        return deepLink;
    }

    /*package*/ void setDeepLink(String deepLink) {
        this.deepLink = deepLink;
    }
    
    
    
}
