package de.dfki.sds.mschroeder.commons.lang;

import de.dfki.sds.mschroeder.commons.lang.kb.KBClass;
import de.dfki.sds.mschroeder.commons.lang.kb.KBIndividual;
import de.dfki.sds.mschroeder.commons.lang.kb.KBResource;
import java.util.LinkedList;
import java.util.Set;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class KnowledgeBaseUtility {
    
    /**
     * Checks for the given resource if other resources have to be created first.
     * @param res the resource to add
     * @param list the list is after the call a sorted KBResource list.
     * @param set is for performance check
     */
    public static void hasToBeCreatedFirst(KBResource res, LinkedList<KBResource> list, Set<KBResource> set) {
        if(set.contains(res))
            return;
        
        list.addFirst(res);
        set.add(res);
        
        //recursive call
        if(res instanceof KBIndividual) {
            KBIndividual indiv = (KBIndividual) res;
            
            for(KBResource type : indiv.getTypes())
                hasToBeCreatedFirst(type, list, set);
            
            for(KBResource prop : indiv.getLiteralProperties().keySet())
                hasToBeCreatedFirst(prop, list, set);
            
            for(KBResource prop : indiv.getObjectProperties().keySet())
                hasToBeCreatedFirst(prop, list, set);
            
            for(Set<KBIndividual> objs : indiv.getObjectProperties().values()) {
                for(KBResource obj : objs)
                    hasToBeCreatedFirst(obj, list, set);
            }
            
        } else if(res instanceof KBClass) {
            KBClass clazz = (KBClass) res;
            
            for(KBResource sub : clazz.getSubClasses())
                hasToBeCreatedFirst(sub, list, set);
            
            for(KBResource sup : clazz.getSuperClasses())
                hasToBeCreatedFirst(sup, list, set);
        }
    }
    
    
    
}
