package de.dfki.sds.mschroeder.commons.lang;

import java.util.Map;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class StatisticUtility {

    //Because commons math does not have it
    public static double harmonicMean(double[] data) {
        double sum = 0.0;

        for (int i = 0; i < data.length; i++) {
            if(Double.isNaN(data[i]))
                return 0;
            
            sum += 1.0 / data[i];
        }
        
        if(sum == 0)
            return 0;
        
        return data.length / sum;
    }
    
    /**
     * Calculates the harmonic mean of some features mentioned by keys.
     * Use in features - to invert (1.0 - value) and * to assign weight, e.g. "-length*3.0".
     * Features have to be in range 0.0 to 1.0.
     * @param features e.g. { "feature1": double-value, ... }
     * @param keys e.g. ["feature1"]
     * @return 
     */
    public static double harmonicMeanOf(Map<String, Object> features, String... keys) {
        //https://en.wikipedia.org/wiki/Harmonic_mean

        if(keys.length == 0)
            return 0;
        
        double sum = 0;
        double weightSum = 0;

        for (String key : keys) {
            
            boolean invert = false;
            if (key.startsWith("-")) {
                key = key.substring(1);
                invert = true;
            }
            
            double weight = 1.0;
            if (key.contains("*")) {
                String[] split = key.split("\\*");
                weight = Double.parseDouble(split[1]);
                key = split[0];
            }
            
            double value;
            if(features.get(key) instanceof Integer) {
                value = (int) features.get(key);
            } else {
                value = (double) features.get(key);
            }
            
            if(invert) {
                value = 1.0 - value;
            }
            
            weightSum += weight;
            
            sum += weight / value;
        }

        return weightSum / sum;
    }
    
    public static double averageOf(Map<String, Object> features, String... keys) {

        double sum = 0;
        double weightSum = 0;

        for (String key : keys) {
            
            boolean invert = false;
            if (key.startsWith("-")) {
                key = key.substring(1);
                invert = true;
            }
            
            double weight = 1.0;
            if (key.contains("*")) {
                String[] split = key.split("\\*");
                weight = Double.parseDouble(split[1]);
                key = split[0];
            }
            
            double value;
            if(features.get(key) instanceof Integer) {
                value = (int) features.get(key);
            } else {
                value = (double) features.get(key);
            }
            
            if(invert) {
                value = 1.0 - value;
            }
            
            weightSum += weight;
            sum += value * weight;
        }

        return sum / weightSum;
    }
    
    public static String cleanFeature(String feature) {
        String key = feature;
         if (key.startsWith("-")) {
            key = key.substring(1);
        }

        if (key.contains("*")) {
            String[] split = key.split("\\*");
            key = split[0];
        }
        return key;
    }

    public static double fscore(double precision, double recall) {
        return 2 * ((precision * recall) / (precision + recall));
    }
}
