package de.dfki.sds.mschroeder.commons.lang;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class StringUtility {

    /**
     * Creates a number based hash.
     * @param s
     * @return 
     */
    public static String hash(String s) {
        int hash = 7;
        for (int i = 0; i < s.length(); i++) {
            hash = hash * 31 + s.charAt(i);
        }
        return String.valueOf(hash);
    }
    
    public static String hashalpha(String s) {
        int hash = 7;
        for (int i = 0; i < s.length(); i++) {
            hash = hash * 31 + s.charAt(i);
        }
        String str = String.valueOf(hash);
        String resp = "";
        for(int i = 0; i < str.length(); i++) {
            String ss = "" + str.charAt(i);
            if(ss.equals("-")) {
                resp += "A";
            } else {
                resp += (char)('a' + Integer.parseInt(ss));
            }
        }
        return resp;
    }

    public static String hashhex6(String s) {
        int hash = 7;
        for (int i = 0; i < s.length(); i++) {
            hash = hash * 31 + s.charAt(i);
        }
        String r = convertByteArrayToHexString(String.valueOf(hash).getBytes());
        return r.substring(0, Math.min(r.length(), 6));
    }

    public static String convertByteArrayToHexString(byte[] arrayBytes) {
        StringBuilder stringBuffer = new StringBuilder();
        for (int i = 0; i < arrayBytes.length; i++) {
            stringBuffer.append(Integer.toString((arrayBytes[i] & 0xff) + 0x100, 16)
                    .substring(1));
        }
        return stringBuffer.toString();
    }
    
    public static String sha256(String str) {
        MessageDigest digest;
        try {
            digest = MessageDigest.getInstance("SHA-256");
        } catch (NoSuchAlgorithmException ex) {
            throw new RuntimeException(ex);
        }
        byte[] encodedhash = digest.digest(str.getBytes(StandardCharsets.UTF_8));
        return convertByteArrayToHexString(encodedhash);
    }
    
    private static MessageDigest md5Digest;
    
    public static String md5(String str) {
        if(md5Digest == null) {
            try {
                md5Digest = MessageDigest.getInstance("MD5");
            } catch (NoSuchAlgorithmException ex) {
                throw new RuntimeException(ex);
            }
        }
        byte[] encodedhash = md5Digest.digest(str.getBytes(StandardCharsets.UTF_8));
        return convertByteArrayToHexString(encodedhash);
    }
    
    /**
     * Calculates a long hash based on prime number and uses each character of the string.
     * @param string
     * @return 
     */
    public static long longHash(String string) {
        long h = 1125899906842597L; // prime
        int len = string.length();

        for (int i = 0; i < len; i++) {
            h = 31 * h + string.charAt(i);
        }
        return h;
    }
    
    /**
     * Checks source contains target for all sources.
     * @param target
     * @param sources
     * @return 
     */
    public static boolean isContainedInAny(String target, String... sources) {
        for(String source : sources) {
            if(source.contains(target))
                return true;
        }
        return false;
    }
    
    public static List<String> getSymbols() {
        List<String> l = new ArrayList<>();
        for(int i = 33; i <= 126; i++) {
            char c = (char)i;
            if(!(Character.isLetter(i) || Character.isDigit(c))) {
                l.add(String.valueOf(c));
            }
        }
        return l;
    }
    
    public static String replaceSymbols(String str, String replaceWith) {
        for(String symbol : getSymbols()) {
            str = str.replace(symbol, replaceWith);
        }
        return str;
    }
    
    public static String splitCamelCaseString(String s) {
        return splitCamelCaseString(s, " ");
    }
    
    public static String splitCamelCaseString(String s, String separator) {
        LinkedList<String> result = new LinkedList<String>();
        for (String w : s.split("(?<!(^|[A-Z]))(?=[A-Z])|(?<!^)(?=[A-Z][a-z])")) {
            result.add(w);
        }
        return result.stream().collect(Collectors.joining(" ")).replaceAll("\\s+", separator);
    }

    //https://stackoverflow.com/a/14424783
    public static String encodeURIComponent(String s) {
        String result;

        try {
            result = URLEncoder.encode(s, "UTF-8")
                    .replace("+", "%20")
                    .replace("%21", "!")
                    .replace("%27", "'")
                    .replace("%28", "(")
                    .replace("%29", ")")
                    .replace("%7E", "~");
                    /*
                    .replaceAll("\\+", "%20")
                    .replaceAll("\\%21", "!")
                    .replaceAll("\\%27", "'")
                    .replaceAll("\\%28", "(")
                    .replaceAll("\\%29", ")")
                    .replaceAll("\\%7E", "~");
                    */
        } catch (UnsupportedEncodingException e) {
            result = s;
        }

        return result;
    }

    public static String decodeURIComponent(String s) {
        String result;

        try {
            result = URLDecoder.decode(s, "UTF-8");
            //TODO
            //.replaceAll("\\+", "%20")
            //.replaceAll("\\%21", "!")
            //.replaceAll("\\%27", "'")
            //.replaceAll("\\%28", "(")
            //.replaceAll("\\%29", ")")
            //.replaceAll("\\%7E", "~");
        } catch (UnsupportedEncodingException e) {
            result = s;
        }

        return result;
    }
    
    public static String makeWhitespaceVisible(String s) {
        if(s == null)
            return null;
        
        if(s.isEmpty())
            return "";
        
        return s.replace(" ", "␣").replace("\n", "\\n").replace("\t", "\\t").replace("\r", "\\r");
    }
    
    public static String makeOneLine(String s) {
        if(s == null)
            return null;
        
        if(s.isEmpty())
            return "";
        
        return s.replace("\n", "\\n").replace("\t", "\\t").replace("\r", "\\r");
    }
    
    /**
     * Use this to split something like "Busfahrplan__115_Uni-(Ost).pdf" into "Busfahrplan 115 Uni Ost pdf"
     * @param s
     * @return 
     */
    public static String symbolsToWhitespace(String s) {
        List<String> symbols = getSymbols();
        for(String symbol : symbols) {
            s = s.replaceAll(RegexUtility.quote(symbol) + "+", " ");
        }
        return s.replaceAll("\\s+", " ");
    }
    
    /**
     * Returns http://.../[localname] or http://...#[localname] of URI.
     * We typically use this in semantic context.
     * @param uri
     * @return 
     */
    public static String getLocalName(String uri) {
        int a = uri.lastIndexOf("/");
        int b = uri.lastIndexOf("#");
        int m = Math.max(a, b);
        if(m == -1)
            return null;
        
        return uri.substring(m+1, uri.length());
    }
    
    public static String toAcronym(String str) {
        return str.replaceAll("\\B.|\\P{L}", "").toUpperCase();
    }
    
    /**
     * Converts umlaut to ascii, e.g. ä to ae.
     * @param text
     * @return 
     */
    public static String umlautConversion(String text) {
        return text
                .replace("Ü", "Ue")
                .replace("Ö", "Oe")
                .replace("Ä", "Ae")
                .replace("ü", "ue")
                .replace("ö", "oe")
                .replace("ä", "ae")
                .replace("ß", "ss");
    }
    
    public static String toProperCase(String s) {
        if (s == null) {
            return null;
        }

        if (s.length() == 0) {
            return s;
        }

        if (s.length() == 1) {
            return s.toUpperCase();
        }

        return s.substring(0, 1).toUpperCase()
                + s.substring(1).toLowerCase();
    }
}
