package de.dfki.sds.mschroeder.commons.lang.data;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * Contains two N2M: One which are persistent (coming from KG) and one which will be created 'stage'.
 * @author Markus Schr&ouml;der
 */
public class DualN2M<N, M> {
    
    //this is the ram data for mapping (and single N or M adds)
    //it is the one which already exists (e.g. in KG)
    protected N2M<N, M> persistent;
    
    //this is the ram data for mapping that are now created with analyze
    protected N2M<N, M> stage;

    public DualN2M() {
        persistent = new N2M<>();
        stage = new N2M<>();
    }
    
    public List<N> getAllNsList() {
        List<N> l = new ArrayList<>();
        l.addAll(persistent.getNList());
        l.addAll(stage.getNList());
        return l;
    }
    
    public List<M> getAllMsList() {
        List<M> l = new ArrayList<>();
        l.addAll(persistent.getMList());
        l.addAll(stage.getMList());
        return l;
    }
    
    public Set<N> getAllNsSet() {
        Set<N> l = new HashSet<>();
        l.addAll(persistent.getNSet());
        l.addAll(stage.getNSet());
        return l;
    }
    
    public Set<M> getAllMsSet() {
        Set<M> l = new HashSet<>();
        l.addAll(persistent.getMSet());
        l.addAll(stage.getMSet());
        return l;
    }
    
    public List<N> getAllNsSorted(Comparator<N> comp) {
        List<N> ns = new ArrayList<>(getAllNsSet());
        ns.sort(comp);
        return ns;
    }
    
    public List<M> getAllMsSorted(Comparator<M> comp) {
        List<M> ms = new ArrayList<>(getAllMsSet());
        ms.sort(comp);
        return ms;
    }
    
    public Set<N> getAllNs(M m) {
        Set<N> ns = new HashSet<>();
        ns.addAll(getPersistent().getNs(m));
        ns.addAll(getStage().getNs(m));
        return ns;
    }
    
    public boolean isPersistentM(M m) {
        return persistent.containsM(m);
    }
    
    public boolean isStageM(M m) {
        return stage.containsM(m);
    }
    
    public boolean isPersistentN(N n) {
        return persistent.containsN(n);
    }
    
    public boolean isStageN(N n) {
        return stage.containsN(n);
    }
    
    public N2M<N, M> getPersistent() {
        return persistent;
    }

    public N2M<N, M> getStage() {
        return stage;
    }
    
}
