package de.dfki.sds.mschroeder.commons.lang.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import static java.util.stream.Collectors.toList;

/**
 * To hold and edit in RAM an n to m mapping.
 * @author Markus Schr&ouml;der
 */
public class N2M<N, M> {
    
    private Map<N, Set<M>> n2m;
    private Map<M, Set<N>> m2n;
    
    public N2M() {
        n2m = new LinkedHashMap<>();
        m2n = new LinkedHashMap<>();
    }
    
    public void addN(N n) {
        n2m.computeIfAbsent(n, p -> new HashSet<>());
    }
    
    public void addM(M m) {
        m2n.computeIfAbsent(m, p -> new HashSet<>());
    }
    
    public void removeN(N n) {
        Set<M> ms = n2m.get(n);
        if(ms == null)
            return;
        
        List<M> l = ms.stream().collect(toList());
        for(M m : l) {
            removeMapping(n, m);
        }
    }
    
    public void removeM(M m) {
        Set<N> ns = m2n.get(m);
        if(ns == null)
            return;
        
        List<N> l = ns.stream().collect(toList());
        for(N n : l) {
            removeMapping(n, m);
        }
    }
    
    public void addMapping(N n, M m) {
        n2m.computeIfAbsent(n, p -> new HashSet<>()).add(m);
        m2n.computeIfAbsent(m, p -> new HashSet<>()).add(n);
    }
    
    public void removeMapping(N n, M m) {
        removeMapping(n, m, n2m);
        removeMapping(m, n, m2n);
    }
    
    public boolean containsN(N n) {
        return n2m.containsKey(n);
    }
    
    public boolean containsM(M m) {
        return m2n.containsKey(m);
    }
    
    private <A,B> void removeMapping(A a, B b, Map<A, Set<B>> map) {
        Set<B> l = map.get(a);
        if(l != null) {
            if(b != null) {
                l.remove(b);
            } else {
                l.clear();
            }
            if(l.isEmpty()) {
                map.remove(a);
            }
        }
    }
    
    public void clear() {
        n2m.clear();
        m2n.clear();
    }
    
    public Set<N> getNSet() {
        return new HashSet<>(n2m.keySet());
    }
    
    public List<N> getNList() {
        return new ArrayList<>(n2m.keySet());
    }
    
    public Set<M> getMSet() {
        return new HashSet<>(m2n.keySet());
    }
    
    public List<M> getMList() {
        return new ArrayList<>(m2n.keySet());
    }
    
    public Set<N> getNs(M m) {
        Set<N> s = m2n.get(m);
        if(s == null)
            return new HashSet<>();
        return s;
    }
    
    public Set<M> getMs(N n) {
        Set<M> s = n2m.get(n);
        if(s == null)
            return new HashSet<>();
        return s;
    }
    
}
