package de.dfki.sds.mschroeder.commons.lang.kb;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class KBIndividual extends KBResource {
    
    private List<KBClass> types;
    private Map<KBProperty, Set<String>> literalProperties;
    private Map<KBProperty, Set<KBIndividual>> objectProperties;

    public KBIndividual() {
    	this(null, null, null);
    }

    public KBIndividual(String uri, String prefLabel, String comment) {
        super(uri, prefLabel, comment);
        types = new ArrayList<>();
        literalProperties = new HashMap<>();
        objectProperties = new HashMap<>();
    }

    public List<KBClass> getTypes() {
        return types;
    }

    public void setTypes(List<KBClass> types) {
        this.types = types;
    }
    
    public void setType(KBClass type) {
        this.types = Collections.singletonList(type);
    }

    public Map<KBProperty, Set<String>> getLiteralProperties() {
        return literalProperties;
    }

    public void setLiteralProperties(Map<KBProperty, Set<String>> literalProperties) {
        this.literalProperties = literalProperties;
    }
    
    public void addLiteralProperty(KBProperty prop, String literal) {
        if (this.literalProperties == null)
            this.literalProperties = new HashMap<>();
        this.literalProperties.computeIfAbsent(prop, p -> new HashSet<>()).add(literal);
    }

    public Map<KBProperty, Set<KBIndividual>> getObjectProperties() {
        return objectProperties;
    }

    public void setObjectProperties(Map<KBProperty, Set<KBIndividual>> objectProperties) {
        this.objectProperties = objectProperties;
    }
    
    public void addObjectProperty(KBProperty prop, KBIndividual object) {
        if (this.objectProperties == null)
            this.objectProperties = new HashMap<>();
        this.objectProperties.computeIfAbsent(prop, p -> new HashSet<>()).add(object);
    }
    
    @Override
    public String toString() {
    	return "(I) " + this.prefLabel + " <" + this.uri + ">";
    }
    
}
