package de.dfki.sds.mschroeder.commons.lang.kb;

import java.util.List;

/**
 * API to only read from knowlege base in order to align entities in workspace with knowledge base.
 * @author Markus Schr&ouml;der
 */
public interface KnowledgeBaseReadAPI {
    
    
    
    /**
     * "Topic", "Markus", "abc123xyz" and "Kaiserslautern" is sent to the knowledge base.
     * It returns for "Topic" the class pimo:thing#Topic, for "Markus" it returns many persons with that name,
     * Because "abc123xyz" is not in the knowledge base the list of concepts is empty.
     * "Kaiserslautern" is mapped to instance "pimo:1444048778655:39".
     * @param texts a list of phrases (multi-word terms).
     * @return a list of the exact size as the input list. The order is the same as the input texts list.
     */
    public List<List<KBResource>> mappingTextToResource(List<String> texts) throws Exception;
    
    public List<List<KBClass>> mappingTextToClass(List<String> texts) throws Exception;
    
    //alignment
    public List<List<KBIndividual>> mappingTextToIndividual(List<String> texts) throws Exception;
    
    public List<List<KBProperty>> mappingTextToProperty(List<String> texts) throws Exception;
    
    
    //gazetteer
    public List<KBIndividual> getIndividualsByType(String typeUri) throws Exception;
    
    
    //einordnen
    public KBClass getClassTree(String rootClassUri) throws Exception;
    
    public KBProperty getPropertyTree(String rootClassUri) throws Exception;
    
    
    
    public String getId();
    public String getDisplayName();
}
