package de.dfki.sds.mschroeder.commons.lang.math;

import java.util.Iterator;
import java.util.StringJoiner;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class CharInterval implements Iterable<Character> {
    
    public static final CharInterval DIGITS = new CharInterval('0', '9');
    
    public static final CharInterval SYM1 = new CharInterval('!', '/');
    public static final CharInterval SYM2 = new CharInterval(':', '@');
    public static final CharInterval SYM3 = new CharInterval('[', '`');
    public static final CharInterval SYM4 = new CharInterval('{', '~');
    public static final CharInterval LETTER_SM = new CharInterval('a', 'z');
    public static final CharInterval LETTER_CAP = new CharInterval('A', 'Z');
    
    private Interval interval;

    public CharInterval(char pos) {
        interval = new Interval(pos);
    } 
    
    public CharInterval(char begin, char end) {
        interval = new Interval(begin, end);
    }

    public boolean isEmpty() {
        return getLength() == 0;
    }
    
    public int getLength() {
        return interval.getLength();
    }
    
    public char getBegin() {
        return (char) interval.getBegin();
    }

    public char getEnd() {
        return (char) interval.getEnd();
    }
    
    public boolean containedInclusive(char pos) {
        return interval.containedInclusive(pos);
    }
    
    //exclusive border
    public boolean containedExclusive(char pos) {
        return interval.containedExclusive(pos);
    }
    
    public static CharInterval create(char begin, char end) {
        return new CharInterval(begin, end);
    }
    
    public static CharInterval create(char pos) {
        return new CharInterval(pos);
    }

    @Override
    public Iterator<Character> iterator() {
        return new Iterator<Character>() {
            
            private Iterator<Integer> iter = interval.iterator();
            
            @Override
            public boolean hasNext() {
                return iter.hasNext();
            }

            @Override
            public Character next() {
                int i = iter.next();
                char c = (char)i;
                return c;
            }
        };
    }

    @Override
    public String toString() {
        StringJoiner sj = new StringJoiner(", ");
        iterator().forEachRemaining(c -> sj.add(String.valueOf("'" + c + "'")));
        return sj.toString();
    }
    
    
    
}
