package de.dfki.sds.mschroeder.commons.lang.math;

import java.util.Iterator;

/**
 *
 * @author Markus Schr&ouml;der
 */
public class Interval implements Iterable<Integer> {
    private int begin;
    private int end;

    public Interval(int pos) {
        begin = pos;
        end = pos;
    } 
    
    public Interval(int begin, int end) {
        this.begin = begin;
        this.end = end;
    }

    public boolean isEmpty() {
        return getLength() == 0;
    }
    
    public int getLength() {
        return end - begin;
    }
    
    public int getBegin() {
        return begin;
    }

    public int getEnd() {
        return end;
    }
    
    public boolean containedInclusive(int pos) {
        return begin <= pos && pos <= end;
    }
    
    //exclusive border
    public boolean containedExclusive(int pos) {
        return begin < pos && pos < end;
    }
    
    public static Interval create(int begin, int end) {
        return new Interval(begin, end);
    }
    
    public static Interval create(int pos) {
        return new Interval(pos);
    }

    @Override
    public Iterator<Integer> iterator() {
        return new Iterator<Integer>() {
            
            private int cur = begin;
            private int e = end;
            
            @Override
            public boolean hasNext() {
                return cur <= e;
            }

            @Override
            public Integer next() {
                if(!hasNext()) {
                    throw new RuntimeException("at end of interval");
                }
                int result = cur;
                cur++;
                return result;
            }
        };
    }
    
    
}
