package de.dfki.sds.mschroeder.commons.lang.math;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * A score has a value and some meta data like label, comment, etc.
 * @author Markus Schr&ouml;der
 */
public class Score<T> implements Comparable<Score<T>> {
    
    private String label;
    private String comment;
    private String explaination;
    private double value;
    private List<Score<T>> parents;

    public Score(String label, String comment, String explaination, double value) {
        this.label = label;
        this.comment = comment;
        this.explaination = explaination;
        this.value = value;
        this.parents = new LinkedList<>();
    }
    
    public Score(String label, String comment, String explaination) {
        this.label = label;
        this.comment = comment;
        this.explaination = explaination;
        this.value = Double.NaN;
        this.parents = new LinkedList<>();
    }

    public Score copyWithValue(double value) {
        return new Score(this.label, this.comment, this.explaination, value);
    }
    
    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getComment() {
        return comment;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public String getExplaination() {
        return explaination;
    }

    public void setExplaination(String explaination) {
        this.explaination = explaination;
    }

    public double getValue() {
        return value;
    }

    public void setValue(double value) {
        this.value = value;
    }

    public List<Score<T>> getParents() {
        return parents;
    }

    public void setParents(List<Score<T>> parents) {
        this.parents = parents;
    }

    public Map<String, Object> toMap() {
        Map jo = new HashMap<>();
        jo.put("label", label);
        jo.put("comment", comment);
        jo.put("explaination", explaination);
        return jo;
    }
    
    @Override
    public String toString() {
        return "Score{" + "label=" + label + ", value=" + value + '}';
    }

    @Override
    public int compareTo(Score<T> other) {
        return this.label.compareTo(other.label);
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 43 * hash + Objects.hashCode(this.label);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Score<?> other = (Score<?>) obj;
        if (!Objects.equals(this.label, other.label)) {
            return false;
        }
        return true;
    }
    
    public String toStringTree() {
        StringBuilder sb = new StringBuilder();
        toStringTree("", true, sb);
        return sb.toString();
    }

    private void toStringTree(String prefix, boolean isTail, StringBuilder sb) {
        sb.append(prefix).append(isTail ? "└── " : "├── ").append(getLabel()).append(" - ").append(getComment()).append("\n");
        for (int i = 0; i < parents.size() - 1; i++) {
            parents.get(i).toStringTree(prefix + (isTail ? "    " : "│   "), false, sb);
        }
        if (parents.size() > 0) {
            parents.get(parents.size() - 1)
                    .toStringTree(prefix + (isTail ?"    " : "│   "), true, sb);
        }
    }
    
}
