package de.dfki.sds.mschroeder.commons.lang.swing;

import de.dfki.sds.mschroeder.commons.lang.MapUtility;
import java.awt.AWTEvent;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Map;

/**
 * Tracks swing-global mouse useage and writes it to json.
 * @author Markus Schr&ouml;der
 */
public class MouseUsageTracker {
    
    //to track the mouse usage
    private double mouseDistance;
    private Map<Integer, Integer> event2count;
    
    private transient AWTEvent lastMouseMove;
    //private transient long begin;
    
    public MouseUsageTracker() {
        registerMouseEventListener();
    }
    
    //to receive the mouse event everywhere
    private void registerMouseEventListener() {
        event2count = new HashMap<>();
        
        long eventMask = AWTEvent.MOUSE_MOTION_EVENT_MASK + AWTEvent.MOUSE_EVENT_MASK;
        Toolkit.getDefaultToolkit().addAWTEventListener((AWTEvent e) -> {
            
            if(e.getID() == MouseEvent.MOUSE_MOVED) {
                if(lastMouseMove != null) {
                    MouseEvent a = (MouseEvent) lastMouseMove;
                    MouseEvent b = (MouseEvent) e;
                    
                    mouseDistance += Point2D.distance(b.getX(), b.getY(), a.getX(), a.getY());
                }
                
                lastMouseMove = e;
            } else {
                MapUtility.fillKey2CountMap(e.getID(), event2count);
            }
            //System.out.println(mouseDistance + " " + event2count);
        }, eventMask);
        
        //begin = System.currentTimeMillis();
    }

    private String getText(int id) {
        switch(id) {
            case MouseEvent.MOUSE_CLICKED: return "clicked";
            case MouseEvent.MOUSE_DRAGGED: return "dragged";
            case MouseEvent.MOUSE_ENTERED: return "entered";
            case MouseEvent.MOUSE_EXITED: return "exited";
            case MouseEvent.MOUSE_MOVED: return "moved";
            case MouseEvent.MOUSE_PRESSED: return "pressed";
            case MouseEvent.MOUSE_RELEASED: return "released";
            case MouseEvent.MOUSE_WHEEL: return "wheel";
        }
        return String.valueOf(id);
    }
    
    public double getMouseDistance() {
        return mouseDistance;
    }

    public Map<Integer, Integer> getEvent2count() {
        return event2count;
    }
    
    /*
    @Deprecated
    public void save() {
        JSONObject stats = new JSONObject();
     
        long end = System.currentTimeMillis();
        stats.put("begin", begin);
        stats.put("end", end);
        long duration = end-begin;
        stats.put("duration", duration);
        stats.put("durationText", DurationFormatUtils.formatDurationWords(duration, true, true));
        
        JSONObject event2countObject = new JSONObject();
        for(Entry<Integer, Integer> e : event2count.entrySet()) {
            event2countObject.put(getText(e.getKey()), e.getValue());
        }
        stats.put("event2count", event2countObject);
        
        stats.put("mouseDistance", mouseDistance);
        
        File outfile = new File("data/" + String.valueOf(System.currentTimeMillis()) + "-usage-stats.json");
        
        try {
            FileUtils.writeStringToFile(outfile, stats.toString(4), "UTF-8");
        } catch (IOException ex) {
            //ignore (not so important)
        }
    }
*/
    
}
